#ifndef CHIDATE_H
#define CHIDATE_H

/****************************************************************************
** COPYRIGHT (C):    1992 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          ChiLib
** FILE:             chidate.h
** PURPOSE:          Date class
** VERSION           1.0
** LANGUAGE:         Borland  C++ 3.0
** TARGET:           IBM PC
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       9-30-93  (CSH)
** UPDATE HISTORY:

2-7-94 CSH Added Date::compare, relational operators

10-3-95 CSH Fixed days_per_month, August has 31 days and September 30

**
****************************************************************************/

#line 20 "chidate.cpp"

#include "chisetup.h"

#line 24 "chidate.cpp"

#include <iostream.h>

#line 33 "chidate.cpp"


class Chi_Date
/* PURPOSE:    Store dates and perform date arithmetic
*/
{
public:
   enum Weekday { MON, TUE, WED, THU, FRI, SAT, SUN };

   Chi_Date();
   Chi_Date(int d, int m, int y); // construct from day, month, year

   void advance(long n); // advance this date by n days

   int day() const;
   int month() const;
   int year() const;
   Weekday weekday() const;
   Chi_Date add_days(long n) const; // the date that is n days away from this date
   long days_between(const Chi_Date& b) const; // number of days between this date and b
   int compare(const Chi_Date& b) const;

   static int compare(const Chi_Date& a, const Chi_Date& b);
   static int hash(const Chi_Date& a);

   Bool operator<(const Chi_Date& b) const;
   Bool operator<=(const Chi_Date& b) const;
   Bool operator==(const Chi_Date& b) const;
   Bool operator!=(const Chi_Date& b) const;
   Bool operator>(const Chi_Date& b) const;
   Bool operator>=(const Chi_Date& b) const;

private:
   Bool is_valid() const;
   static Bool is_leap(int year);

   int _day;
   int _month;
   int _year;

   static int days_per_month[12];
};


#line 162 "chidate.cpp"
extern istream& operator>>(istream& is, Chi_Date& date);

#line 187 "chidate.cpp"
extern ostream& operator<<(ostream& os, const Chi_Date& date);

#line 312 "chidate.cpp"


inline int Chi_Date::day() const
/* RETURNS:    the day of this date
*/
{  return _day;
}

/*.........................................................................*/

inline int Chi_Date::month() const
/* RETURNS:    the month of this date
*/
{  return _month;
}

/*.........................................................................*/

inline int Chi_Date::year() const
/* RETURNS:    the year of this date
*/
{  return _year;
}

/*.........................................................................*/

inline Bool Chi_Date::operator<(const Chi_Date& b) const
{  return compare(b) < 0;
}

/*.........................................................................*/

inline Bool Chi_Date::operator<=(const Chi_Date& b) const
{  return compare(b) <= 0;
}

/*.........................................................................*/

inline Bool Chi_Date::operator==(const Chi_Date& b) const
{  return compare(b) == 0;
}

/*.........................................................................*/

inline Bool Chi_Date::operator!=(const Chi_Date& b) const
{  return compare(b) != 0;
}

/*.........................................................................*/

inline Bool Chi_Date::operator>(const Chi_Date& b) const
{  return compare(b) > 0;
}

/*.........................................................................*/

inline Bool Chi_Date::operator>=(const Chi_Date& b) const
{  return compare(b) >= 0;
}


#endif
