#ifndef CHIGCBGI_H
#define CHIGCBGI_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          ChiLib
** FILE:             chigcbgi.h
** PURPOSE:          Graphics context for BGI graphics
** VERSION           1.0
** LANGUAGE:         Borland  C++ 3.1/4.0
** TARGET:           IBM PC
** PROGRAMMER:       Cay Horstmann (CSH)
** START DATE:       2-19-94  (CSH)
** UPDATE HISTORY:
**
****************************************************************************/

#line 19 "chigcbgi.cpp"

#include "chisetup.h"

#line 32 "chigcbgi.cpp"

#ifndef CHISTRNG_H
#include "chistrng.h"
#endif

#line 37 "chigcbgi.cpp"
extern int chi_main(int, char**);

#line 41 "chigcbgi.cpp"


class Chi_GraphicsContext;

class Chi_MouseEvent
/* PURPOSE:    a mouse click or move
*/
{
public:
   enum Type { NOT_INSTALLED, LBUTTONDOWN, MOUSEMOVE, LBUTTONUP };
   Chi_MouseEvent(Type, double, double);

   Type type() const { return _type; }
   double x() const { return _x; }
   double y() const { return _y; }
private:
   Type _type;
   double _x;
   double _y;
};

/*-------------------------------------------------------------------------*/

class Chi_Mouse
/* PURPOSE: Interfaces with mouse to get mouse position and events
*/
{
public:
   Chi_Mouse();

   Chi_MouseEvent read(const Chi_GraphicsContext& gc);

   Bool is_installed() const;

   int get_position(int&, int&) const;
   void show_cursor();
   void hide_cursor();


private:
   enum
   {  MOUSE_INTR = 0x33,
      RETF = 0xCF, // far return instruction
      SHOWCURSOR = 1,
      HIDECURSOR = 2,
      READMOUSE = 3,
   };
   Bool _installed;
};

/*-------------------------------------------------------------------------*/

class Chi_GraphicsContext
/* PURPOSE: Sends graphics primitives to a display window
*/
{
public:
   enum Color
   {  BLACK, BLUE, GREEN, CYAN,
      RED, MAGENTA, BROWN, LIGHTGRAY, /* dark colors */
      DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN,
      LIGHTRED, LIGHTMAGENTA, YELLOW, WHITE /* light colors */,
      DEFAULT_COLOR = WHITE
   };
   enum PenStyle { NORMAL_PEN, THICK_PEN, DOTTED_PEN, DASHED_PEN };
   enum BrushStyle { EMPTY_BRUSH, SOLID_BRUSH,
      HORIZ_BRUSH, FDIAG_BRUSH, BDIAG_BRUSH, XDIAG_BRUSH };
   enum FontStyle { SYS_FONT, SANS_FONT, ROMAN_FONT };
   enum RopMode { COPY_ROP, XOR_ROP, AND_ROP, NOT_ROP, OR_ROP };

   Chi_GraphicsContext();
   void close(Bool wait = TRUE);

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void arc(double x, double y, double ra, double rb, double x1, double y1,
      double x2, double y2);
   void rectangle(double xleft, double ytop, double xright, double ybot);
   void ellipse(double x, double y, double ra, double rb);
   void piesegment(double x, double y, double ra, double rb, double x1, double y1,
      double x2, double y2);

   void text(Chi_String t, double x, double y);
   void text_extent(Chi_String t, double& x, double& y);

   void set_brush_style(BrushStyle);
   void set_brush_color(Color);
   void set_pen_style(PenStyle);
   void set_pen_color(Color);
   void set_font_size(double s);
   void set_font_style(FontStyle);
   void set_rop_mode(RopMode);

   void set_window_coord(double xmin, double ymin, double xmax, double ymax);
   void set_clip_rectangle(double xmin, double ymin, double xmax, double ymax);
   void clear();
   void mouse_events(int (*h)(const Chi_MouseEvent&, void*), void* p);

private:
   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
   int user_to_disp_dx(double x) const;
   int user_to_disp_dy(double y) const;

   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;

   double _user_xmin;
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;

   int _disp_xmax;
   int _disp_ymax;
   int _disp_xoff;
   int _disp_yoff;

   BrushStyle _brush;
   FontStyle _font;
   double _font_size;

   Chi_Mouse _mouse;
friend class Chi_Mouse;
};


#endif
