#ifndef CHIGCWIN_H
#define CHIGCWIN_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          ChiLib
** FILE:             chigcwin.h
** PURPOSE:          Graphics context class (Windows version)
** VERSION           1.0
** LANGUAGE:         Borland  C++ 3.1/4.0
** TARGET:           IBM PC
** PROGRAMMERS:      Cay Horstmann (CSH)
**                   Charles Miller (CBM)
** START DATE:       4-25-94  (CBM)
** UPDATE HISTORY:
9/27/95 CSH 			changed user_to_disp_dx -> user_to_disp_x in
							rectangle and set_clip_rectangle
****************************************************************************/

#line 21 "chigcwin.cpp"

#include "chisetup.h"

#line 25 "chigcwin.cpp"

#include <windows.h>

#line 33 "chigcwin.cpp"

#ifndef CHISTRNG_H
#include "chistrng.h"
#endif

#line 37 "chigcwin.cpp"
extern int chi_main(int, char**);

#line 51 "chigcwin.cpp"


class Chi_MouseEvent
{
public:
   enum Type { NOT_INSTALLED, LBUTTONDOWN, MOUSEMOVE, LBUTTONUP };
   Chi_MouseEvent(Type, double, double);

   Type type() const { return _type; }
   double x() const { return _x; }
   double y() const { return _y; }
private:
   Type _type;
   double _x;
   double _y;
};

/*-------------------------------------------------------------------------*/

class Chi_Bitmap
{
public:
   Chi_Bitmap();
   void init(HWND hwnd);
   void close() const;
   void repaint(PAINTSTRUCT&) const;
   void clear_bitmap(COLORREF color) const;
   HDC hdc() const;

private:
   HDC _hdc;                     // a long-lived device context
   HBITMAP _hbm, _hbm_saved;     // handles to the created bitmap objects
   int _disp_xmax;               // width of the bitmap (in bits)
   int _disp_ymax;               // height of the bitmap (in bits)
};

/*-------------------------------------------------------------------------*/

class Chi_GraphicsContext
// PURPOSE: Sends graphics primitives to a display window
{
public:
   enum Color
   {  BLACK, BLUE, GREEN, CYAN,
      RED, MAGENTA, BROWN, LIGHTGRAY,  /* dark colors */
      DARKGRAY, LIGHTBLUE, LIGHTGREEN, LIGHTCYAN,
      LIGHTRED, LIGHTMAGENTA, YELLOW, WHITE  /* light colors */,
      DEFAULT_COLOR = BLACK 
         /* default color for drawing--opposite from background */
   };
   enum PenStyle { NORMAL_PEN, THICK_PEN, DOTTED_PEN, DASHED_PEN };
   enum BrushStyle { EMPTY_BRUSH, SOLID_BRUSH, HORIZ_BRUSH, FDIAG_BRUSH,
                        BDIAG_BRUSH, XDIAG_BRUSH };
   enum FontStyle { SYS_FONT, SANS_FONT, ROMAN_FONT };
   enum RopMode { COPY_ROP, XOR_ROP, OR_ROP, AND_ROP, NOT_ROP };

   Chi_GraphicsContext();
   void close(Bool wait = TRUE);

   void point(double x, double y);
   void line(double xfrom, double yfrom, double xto, double yto);
   void arc(double x, double y, double ra, double rb, double x1, double y1,
      double x2, double y2);
   void rectangle(double xleft, double ytop, double xright, double ybot);
   void ellipse(double x, double y, double ra, double rb);
   void piesegment(double x, double y, double ra, double rb, double x1, double y1,
      double x2, double y2);

   void text(Chi_String t, double x, double y);
   void text_extent(Chi_String t, double& x, double& y);

   void set_brush_style(BrushStyle style);
   void set_brush_color(Color color);
   void set_pen_style(PenStyle style);
   void set_pen_color(Color color);
   void set_font_size(double size);
   void set_font_style(FontStyle style);
   void set_rop_mode(RopMode mode);

   void set_window_coord(double xmin, double ymin, double xmax, double ymax);
   void set_clip_rectangle(double xmin, double ymin, double xmax, double ymax);
   void clear();
   void mouse_events(int (*h)(const Chi_MouseEvent&, void*), void* p);

private:
   COLORREF win_color(Color index) const;

   int user_to_disp_x(double x) const;
   int user_to_disp_y(double y) const;
	int user_to_disp_dx(double x) const;
	int user_to_disp_dy(double y) const;
   double disp_to_user_dx(int x) const;
   double disp_to_user_dy(int y) const;

   double _user_xmin; // the user's window's logical window coords
   double _user_xmax;
   double _user_ymin;
   double _user_ymax;
   int _disp_xmax; // the physical window dimension (in pixels)
   int _disp_ymax;
   Color _brush_color;
   BrushStyle _brush_style;
   Color _pen_color;
   PenStyle _pen_style;
   FontStyle _font_style;
   int _win_rop_mode; // the Windows raster op mode
   int _def_fontheight; // the font height to make 1 unit characters

   HBRUSH _hbrush; // currently selected brush
   HPEN  _hpen;
   LOGFONT _lf; // for maintaining font status
   HFONT _hfont;

   Chi_Bitmap& _bm; // the bitmap to draw on
   HDC _hdc; // device context to the display
};


#endif
