#ifndef GEDFRMWK_H
#define GEDFRMWK_H

/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Mastering Object-Oriented Design
** FILE:             gedfrmwk.h
** PURPOSE:          Graphics editor framework
** VERSION           1.0
** LANGUAGE:         Borland C++ 3.1
** TARGET:           IBM PC
** PROGRAMMERS:      Cay S. Horstmann (CSH)
** START DATE:       3/21/94
** UPDATE HISTORY:
****************************************************************************/

#line 16 "GEDFRMWK.cpp"

#include "chisetup.h"

#line 22 "GEDFRMWK.cpp"

#ifndef CHIARRAY_H
#include "chiarray.h"
#endif

#line 24 "GEDFRMWK.cpp"

#ifndef CHISTRNG_H
#include "chistrng.h"
#endif

#line 26 "GEDFRMWK.cpp"

#ifndef CHISHAPE_H
#include "chishape.h"
#endif

#line 33 "GEDFRMWK.cpp"


class Node
/* PURPOSE:    An abstract class for a node in a graph
*/
{
public:
   virtual Chi_String name() const = 0;
   virtual void plot(Chi_GraphicsContext& gc) const = 0;
   virtual void plot_icon(Chi_Rectangle r, Chi_GraphicsContext& gc) const = 0;
   virtual Node* clone() const = 0;
   virtual Chi_Rectangle enclosing_rect() const = 0;
   virtual Bool is_inside(Chi_Point p) const = 0;
   virtual Chi_Point boundary_point(Chi_Point exterior) const = 0;
   virtual void read(istream& is) = 0;
   virtual void print(ostream& os) const  = 0;

   void move(Chi_Point p);
   Chi_Point center() const;

private:
   Chi_Point _center;
};

/*-------------------------------------------------------------------------*/

class Edge
/* PURPOSE:    An abstract class for an edge joining two nodes
*/
{
public:
   void connect(Node* f, Node* t);

   virtual Chi_String name() const = 0;
   virtual void plot(Chi_GraphicsContext& gc) const = 0;
   virtual void plot_icon(Chi_Rectangle r, Chi_GraphicsContext& gc) const = 0;
   virtual Edge* clone() const = 0;
   virtual Bool is_on(Chi_Point p) const = 0;
   virtual void read(istream& is) = 0;
   virtual void print(ostream& os) const = 0;

   const Node* from() const;
   const Node* to() const;

private:
   Node* _from;
   Node* _to;
};

/*-------------------------------------------------------------------------*/

class Diagram
/* PURPOSE:    A collection of nodes and edges
*/
{
public:
   void add_node(Node*);
   void add_edge(Edge*);
   void remove(Chi_Point);
   Node* find_node(Chi_Point);
   int find_node(const Node*) const;
   Edge* find_edge(Chi_Point) const;
   int search_node(Chi_Point p) const;
   void plot(Chi_GraphicsContext&) const;
   void print(ostream& os) const;
private:
   Chi_PtrArray<Node> _nodes;
   Chi_PtrArray<Edge> _edges;
};

/*-------------------------------------------------------------------------*/

class DiagramEditor
/* PURPOSE:    An editor for diagrams made from nodes and edges
*/
{
public:
   DiagramEditor();
   enum Mode { EXIT, MOVE, ERASE, NODE, EDGE };

   virtual void register_types() = 0;
   void run(Chi_String infile);

   void register_node(Node*);
   void register_edge(Edge*);

   void read(istream& is);
   Node* make_node(Chi_String) const;
   Edge* make_edge(Chi_String) const;

   void event_loop();
   void rubber_band();
   void display();

   void plot_buttons();
   void plot_exit_button(Chi_Rectangle);
   void plot_move_button(Chi_Rectangle);
   void plot_erase_button(Chi_Rectangle);
   void mark_current_button();
   Node* clone_current_node();
   Edge* clone_current_edge();
   Mode click_button(Chi_Point p);
   Mode mode() const;

   int mouse_handler(const Chi_MouseEvent& e);
   static int mouse_handler(const Chi_MouseEvent& e, void* p);

private:

   Chi_PtrArray<Node> _node_types;
   Chi_PtrArray<Edge> _edge_types;

   Diagram _diagram;

   Chi_Rectangle _button_rect;
   int _current_button;

   Node* _start_node;
   Chi_Point _start_point;
   Chi_Point _current_point;

   Chi_GraphicsContext _gc;
   Chi_String _filename;
};


#endif
