/****************************************************************************
** COPYRIGHT (C):    1994 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Mastering Object-Oriented Design
** FILE:             grstream.cpp
** PURPOSE:          Stream with output onto a graphics context
** VERSION           1.0
** LANGUAGE:         Borland C++ 3.1
** TARGET:           IBM PC
** PROGRAMMERS:      Cay S. Horstmann (CSH)
** START DATE:       September 14, 1994
** UPDATE HISTORY:
****************************************************************************/

/* IMPORT ******************************************************************/

#ifndef EXPORT
#include "chisetup.h"
#endif

#include <iostream.h>

EXPORT
#include "chigc.h"
#include "grstream.h"

/* TYPES *******************************************************************/

#ifndef EXPORT

class GraphicStreambuf : public streambuf
/* PURPOSE:    a stream buffer that sends its overflow to a graphics context
*/
{
public:
   GraphicStreambuf(Chi_GraphicsContext& gc);
protected:
   virtual int overflow(int c);
   virtual int underflow() { return 0; }
private:
   Chi_GraphicsContext& _gc;

   double _xnext;
   double _ynext;
};

/*-------------------------------------------------------------------------*/

class GraphicStream : public ostream
/* PURPOSE:    an output stream that sends characters to a graphics context
*/
{
public:
   GraphicStream(Chi_GraphicsContext&);
private:
   GraphicStreambuf _buffer;
};

#endif

/* FUNCTIONS ***************************************************************/

GraphicStreambuf::GraphicStreambuf(Chi_GraphicsContext& gc)
/* RECEIVES:   gc - graphics context for display
*/
: _gc(gc), _xnext(0), _ynext(1)
{  gc.set_font_style(Chi_GraphicsContext::SANS_FONT);
   gc.set_font_size(1.0);
}

/*.........................................................................*/

int GraphicStreambuf::overflow(int c)
/* PURPOSE:    place a character that overflowed from the buffer
   RECEIVES:   c - the character to place
*/
{  char buf[2];
   buf[1] = 0;
   double x, y;
   switch(c)
   {  case EOF:
         break;
      case '\n':
         buf[0] = 'X';
         _gc.text_extent(buf, x, y);
         _xnext = 0;
         _ynext += y;
         break;
      default:
         buf[0] = c;
         _gc.text(buf, _xnext, _ynext); // CSH add _xnext, _ynext
         _gc.text_extent(buf, x, y);
         _xnext += x;
         break;
   }
   return 1;
}

/*-------------------------------------------------------------------------*/

GraphicStream::GraphicStream(Chi_GraphicsContext& gc)
/* RECEIVES:   gc - graphics context for display
*/
:  _buffer(gc),
   ostream(&_buffer)
{}

/* TEST ********************************************************************/

#if defined(GRSTREAM_TEST) || defined(grstream_TEST)

#include <iomanip.h>
#include <math.h>

int chi_main(int, char**)
{  Chi_GraphicsContext gc;
   GraphicStream gs(gc);
   gs << "Hello, World" << endl
      << setiosflags(ios::fixed)
      << setprecision(8) << -M_PI;
   gc.close();
   return 0;
}

#endif

/***************************************************************************/
